#!/bin/bash

set -e

CONFIGURATION="kafkorama-gateway.conf"
JAR="kafkorama-gateway.jar"
#
# load syscofig configuration if not provided otherwise (e.g. via cloud specific environment scripts)
#
if [[ -z "${KAFKORAMA_GATEWAY_EXTRA_OPTS}" ]]; then
    # define max socket descriptors
    if [ -z "${KAFKORAMA_GATEWAY_MAX_DESCRIPTORS}" ]; then
        KAFKORAMA_GATEWAY_MAX_DESCRIPTORS=1000000
    fi
fi

if [[ ! -z ${KAFKORAMA_GATEWAY_EXTRA_OPTS} ]]; then
    KAFKORAMA_GATEWAY_OUTDIR=$(echo ${KAFKORAMA_GATEWAY_EXTRA_OPTS} | awk -v FS='[:space:]*-D' '{for (k=2;k<=NF;k++) print $k}' | grep -i "LogFolder[[:space:]]*=" | tail -1 | cut -d= -f2 | tr -d [:space:]\'\")
    KAFKORAMA_GATEWAY_MEMORY_MB=$(echo ${KAFKORAMA_GATEWAY_EXTRA_OPTS} | awk -v FS='[:space:]*-D' '{for (k=2;k<=NF;k++) print $k}' | grep -i "Memory[[:space:]]*=" | tail -1 | cut -d= -f2 | tr -d [:space:]\'\"[:alpha:])
fi

#
# define outdir
#
if [ -z "${KAFKORAMA_GATEWAY_OUTDIR}" ]; then
    LOGFOLDER=`cat $CONFIGURATION | grep -i "^[[:space:]]*LogFolder[[:space:]]*="`
    KAFKORAMA_GATEWAY_OUTDIR=`awk -F= '{print $2}' <<< $LOGFOLDER | tr -d "[:space:]"`
fi

if [ -z "${KAFKORAMA_GATEWAY_OUTDIR}" ]; then
    KAFKORAMA_GATEWAY_OUTDIR=logs
fi

if [ ! -d ${KAFKORAMA_GATEWAY_OUTDIR}/all ]; then
    mkdir -p ${KAFKORAMA_GATEWAY_OUTDIR}/all && chmod -R 755 ${KAFKORAMA_GATEWAY_OUTDIR}
fi

LOGNOHUP="${KAFKORAMA_GATEWAY_OUTDIR}/all/out.log"
echo "Using log folder: ${KAFKORAMA_GATEWAY_OUTDIR}" | tee -a $LOGNOHUP

if [ ! -z "${KAFKORAMA_GATEWAY_MAX_DESCRIPTORS}" ] && [ $(id -u) -eq 0 ]; then
    echo "Setting the maximum number of file descriptors to ${KAFKORAMA_GATEWAY_MAX_DESCRIPTORS}" | tee -a $LOGNOHUP
    ulimit -n ${KAFKORAMA_GATEWAY_MAX_DESCRIPTORS} || true
fi

#
# check if java is installed and verify its version compatibility with Kafkorama gateway
#
if command -v java 1>/dev/null; then
    echo "Using Java from `command -v java`" | tee -a $LOGNOHUP
else
    echo "Error: Could not find the 'java' executable, is Java installed?" | tee -a $LOGNOHUP && exit 1
fi

JAVA_VERSION=`java -version 2>&1 | sed -E -n 's/.* version "([0-9]*).*$/\1/p'`
if [[ "$JAVA_VERSION" -eq "1" ]]; then
        JAVA_VERSION=`java -version 2>&1 | sed -E -n 's/.* version "1\.([0-9]*).*$/\1/p'`
fi
if [[ ! "$JAVA_VERSION" -ge "8" ]] ; then
        echo "Error: Unsupported java version: \"${JAVA_VERSION}\". Please install Java Runtime Environment (JRE) version 8 (recommended) or later." | tee -a $LOGNOHUP && exit 1
fi

#
# define default JAVA GC options if not provided via KAFKORAMA_GATEWAY_JAVA_GC_OPTS
#
if [ -z "${KAFKORAMA_GATEWAY_JAVA_GC_LOG_OPTS}" ]; then
    if [[ "$JAVA_VERSION" -eq "8" ]] ; then
        KAFKORAMA_GATEWAY_JAVA_GC_LOG_OPTS="-XX:+PrintCommandLineFlags -XX:+PrintGCDetails -XX:+PrintHeapAtGC -XX:+PrintGCTimeStamps -XX:+PrintGCDateStamps -XX:+UseGCLogFileRotation -XX:GCLogFileSize=200M -XX:NumberOfGCLogFiles=5 -Xloggc:${KAFKORAMA_GATEWAY_OUTDIR}/all/gc-%t.log"
    else
        KAFKORAMA_GATEWAY_JAVA_GC_LOG_OPTS="-XX:+PrintCommandLineFlags -Xlog:gc*,gc=debug,gc+phases=debug,gc+heap=debug:file=${KAFKORAMA_GATEWAY_OUTDIR}/all/gc-%t.log:time,uptimemillis:filecount=5,filesize=200M"
    fi
fi

if [ -z "${KAFKORAMA_GATEWAY_JAVA_GC_OPTS}" ]; then
    if [[ "$JAVA_VERSION" -eq "8" ]] ; then
        KAFKORAMA_GATEWAY_JAVA_GC_OPTS="-verbosegc -XX:+UseParallelOldGC"
    else
        KAFKORAMA_GATEWAY_JAVA_GC_OPTS="-verbosegc"
    fi
fi

#
# define the amount of memory to be used by the Kafkorama gateway
#
if [ -z "${KAFKORAMA_GATEWAY_MEMORY_MB}" ]; then
    MEMORY=`cat ${CONFIGURATION} | grep -i "^[[:space:]]*Memory[[:space:]]*=[[:space:]]*[[:digit:]]*[[:space:]]*MB[[:space:]]*$" | tail -1`
    if [ -n "$MEMORY" ]; then
        KAFKORAMA_GATEWAY_MEMORY_MB=`awk -F= '{print $2}' <<< $MEMORY | tr -d "[:space:][:alpha:]"`
    fi
fi

if [ -z "${KAFKORAMA_GATEWAY_MEMORY_MB}" ]; then
        echo "No memory configured." | tee -a $LOGNOHUP
        echo "Please configure parameter Memory in kafkorama-gateway.conf, or configure Memory as option of KAFKORAMA_GATEWAY_EXTRA_OPTS!" | tee -a $LOGNOHUP
        exit 1
fi

echo "Configured memory ${KAFKORAMA_GATEWAY_MEMORY_MB} MB" | tee -a $LOGNOHUP

#
# define Java heap options
#
if [ -z "${KAFKORAMA_GATEWAY_JAVA_HEAP_OPTS}" ]; then
    KAFKORAMA_GATEWAY_JAVA_HEAP_OPTS="-Xms${KAFKORAMA_GATEWAY_MEMORY_MB}M -Xmx${KAFKORAMA_GATEWAY_MEMORY_MB}M"
fi

#
# define all Java options
#
KAFKORAMA_GATEWAY_JAVA_OPTS="-showversion ${KAFKORAMA_GATEWAY_JAVA_HEAP_OPTS} ${KAFKORAMA_GATEWAY_JAVA_GC_OPTS} ${KAFKORAMA_GATEWAY_JAVA_GC_LOG_OPTS} ${KAFKORAMA_GATEWAY_JAVA_EXTRA_OPTS}"

#
# define all Kafkorama gateway options
#
KAFKORAMA_GATEWAY_OPTS="-Dfile.encoding=UTF-8 -Dlog=/dev/null -Dconfig=${CONFIGURATION} ${KAFKORAMA_GATEWAY_EXTRA_OPTS}"

#
# start the Kafkorama gateway
#
echo "[`date`] Starting Kafkorama Gateway ..." | tee -a $LOGNOHUP
CMD="java ${KAFKORAMA_GATEWAY_JAVA_OPTS} ${KAFKORAMA_GATEWAY_OPTS} ${KAFKORAMA_GATEWAY_KAFKA_EXTRA_OPTS} -jar ${JAR} $@"
echo "[`date`] Using command: ${CMD}" | tee -a $LOGNOHUP

exec > >(tee -a $LOGNOHUP) 2>&1
exec ${CMD}

